from nonebot import get_driver, on_command, on_message, require, permission, get_bot
from nonebot.rule import to_me
from nonebot.log import logger
from nonebot.rule import to_me, regex

from nonebot.matcher import Matcher
from nonebot.adapters.onebot.v11 import Message, MessageSegment, MessageEvent
from nonebot.adapters.onebot.v11.permission import GROUP_ADMIN, GROUP_OWNER, PRIVATE_FRIEND, GROUP_MEMBER
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from nonebot.params import CommandArg


from .config import Config

from .biliStream import *
from .biliVideo import *
from .biliTelegram import *
from .basicFunc import *
from .rule import groupMessageRule, privateMessageRule

import os
import json
import sys
import re
from typing import Union

global_config = get_driver().config
config = Config.parse_obj(global_config)

__PLUGIN_NAME = "[B站整合]"

ALL_PERMISSION = GROUP_ADMIN | GROUP_OWNER | PRIVATE_FRIEND

followStreamerCommand = on_command("关注主播", permission=ALL_PERMISSION)
@followStreamerCommand.handle()
async def followStreamerCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    uidList = args.extract_plain_text().split()
    if isinstance(event, PrivateMessageEvent):
        successList, failList = await FollowStreamers(event, event.sender.user_id, uidList, 0)
    if isinstance(event, GroupMessageEvent):
        successList, failList = await FollowStreamers(event, event.group_id, uidList, 1)
    await followStreamerCommand.finish(f"关注成功:\n{successList}\n关注失败:\n{failList}")

unfollowStreamerCommand = on_command("取关主播", aliases={"切割主播"}, permission=ALL_PERMISSION)
@unfollowStreamerCommand.handle()
async def unfollowStreamerCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    uidList = args.extract_plain_text().split()
    if isinstance(event, PrivateMessageEvent):
        successList, failList = await UnfollowStreamers(event, event.sender.user_id, uidList, 0)
    if isinstance(event, GroupMessageEvent):
        successList, failList = await UnfollowStreamers(event, event.group_id, uidList, 1)
    await unfollowStreamerCommand.finish(f"取关成功:\n{successList}\n取关失败:\n{failList}")

listFollowingCommand = on_command("查询关注", aliases={"查询成分"}, permission=ALL_PERMISSION)
@listFollowingCommand.handle()
async def listFollowingCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    inputArgs = args.extract_plain_text().split()
    defaultArgs = ['直播', 'up主', '番剧']

    if isinstance(event, PrivateMessageEvent):
        userID = event.sender.user_id
        userFile = f"{PackagePath}/file/user/{userID}.json"
    elif isinstance(event, GroupMessageEvent):
        groupID = event.group_id
        userFile = f"{PackagePath}/file/group/{groupID}.json"
    
    exceptArgs = set(inputArgs) - set(defaultArgs)
    if len(exceptArgs) != 0:
        logger.info(f'{__PLUGIN_NAME}查询失败，存在错误参数: {exceptArgs}')
        await listFollowingCommand.finish(f"查询失败，存在错误参数:{exceptArgs}\n请正确输入命令,例如: '查询成分 直播' 或 '查询成分 直播 up主 番剧'")
    
    if os.path.exists(userFile):
        with open(userFile, "r+") as f:
            userInfo = json.load(f)

        if not inputArgs:
            inputArgs = defaultArgs
        if '直播' in inputArgs:
            if userInfo[1]:
                textMsg = '关注的主播:\n'
                for info in userInfo[1]:
                    textMsg += '> ' + info + '\n'
            else:
                textMsg = '无关注的主播'
            await listFollowingCommand.send(textMsg)
        if 'up主' in inputArgs:
            if userInfo[2]:
                textMsg = '关注的up主\n'
                for info in userInfo[2]:
                    textMsg += '> ' + info + '\n'
            else:
                textMsg = '无关注的up主'
            await listFollowingCommand.send(textMsg)
        if '番剧' in inputArgs:
            if userInfo[3]:
                textMsg = '关注的番剧\n'
                for info in userInfo[3]:
                    textMsg += '> '+ info + '\n'
            else:
                textMsg = '无关注的番剧'
            await listFollowingCommand.send(textMsg)
        await listFollowingCommand.finish()
    else:
        logger.info(f'{__PLUGIN_NAME}用户文件不存在，准备创建')
        name = event.sender.nickname
        if isinstance(event, GroupMessageEvent):
            bot = get_bot()
            groupInfo = await bot.get_group_info(group_id=id)
            name = groupInfo["group_name"]
        await createUserFile(userFile, nickName=name)
        await listFollowingCommand.finish("关注列表为空")

followUpCommand = on_command("关注up", permission=ALL_PERMISSION)
@followUpCommand.handle()
async def followUpCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    uidList = args.extract_plain_text().split() 
    if isinstance(event, PrivateMessageEvent):
        successList, failList = await FollowUp(event, event.sender.user_id, uidList, 0)
    if isinstance(event, GroupMessageEvent):
        successList, failList = await FollowUp(event, event.group_id, uidList, 1)
    await followUpCommand.finish(f"关注成功:\n{successList}\n关注失败:\n{failList}")

unfollowUpCommand = on_command("取关up", permission=ALL_PERMISSION)
@unfollowUpCommand.handle()
async def unfollowUpCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    uidList = args.extract_plain_text().split()
    if isinstance(event, PrivateMessageEvent):
        successList, failList = await UnfollowUp(event, event.sender.user_id, uidList, 0)
    if isinstance(event, GroupMessageEvent):
        successList, failList = await UnfollowUp(event, event.group_id, uidList, 1)
    await unfollowUpCommand.finish(f"取关成功:\n{successList}\n取关失败:\n{failList}")

followTelegramCommand = on_command("关注番剧", permission=ALL_PERMISSION)
@followTelegramCommand.handle()
async def followTelegramCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    epIDList = args.extract_plain_text().split()
    if isinstance(event, PrivateMessageEvent):
        successList, failList = await FollowTelegram(event, event.sender.user_id, epIDList, 0)
    if isinstance(event, GroupMessageEvent):
        successList, failList = await FollowTelegram(event, event.group_id, epIDList, 1)
    await followTelegramCommand.finish(f"关注成功:\n{successList}\n关注失败:\n{failList}")

unfollowTelegramCommand = on_command("取关番剧", permission=ALL_PERMISSION)
@unfollowTelegramCommand.handle()
async def unfollowTelegramCommandHandler(event: Union[PrivateMessageEvent, GroupMessageEvent], args: Message = CommandArg()):
    seasonIDs = args.extract_plain_text().split()
    if isinstance(event, PrivateMessageEvent):
        successList, failList = await UnfollowTelegram(event, event.sender.user_id, seasonIDs, 0)
    if isinstance(event, GroupMessageEvent):
        successList, failList = await UnfollowTelegram(event, event.group_id, seasonIDs, 1)
    await unfollowTelegramCommand.finish(f"取关成功:\n{successList}\n取关失败:\n{failList}")

followUpByShare = on_message(
    rule=regex('\[CQ:json,[\w\W]*"appid":100951776[\w\W]*space.bilibili.com[\w\W]*[\w\W]*\]') & privateMessageRule, 
    permission=PRIVATE_FRIEND
    )
@followUpByShare.handle()
async def upShareHandler(event: PrivateMessageEvent):
    '''响应用户分享up主空间连接

    Args:
        event (PrivateMessageEvent): 消息事件
    '''

    sJson = event.get_message()[-1].get('data')
    data = json.loads(sJson['data'])
    uid = data['meta']['news']['jumpUrl'].split('?')[0].split('/')[-1]
    
    successList, failList = await FollowUp(event, event.sender.user_id, [int(uid)], 0)
    
    if successList:
        await followUpByShare.finish(f"关注up成功: <{successList[0]}>")
    elif failList:
        await followUpByShare.finish(f"关注up失败: <{failList[0]}> ")

followStreamerByShare = on_message(
    rule=regex('^\[CQ:json,[\w\W]*"appid":100951776[\w\W]*live.bilibili.com[\w\W]*') & privateMessageRule,
    permission=PRIVATE_FRIEND
    )
@followStreamerByShare.handle()
async def streamerShareHandler(event: PrivateMessageEvent):
    '''响应个人用户的直播间分享

    Args:
        event (PrivateMessageEvent): 消息事件
    '''
    sJson = event.get_message()[-1].get('data')
    data = json.loads(sJson['data'])
    roomNumber = data['meta']['news']['jumpUrl'].split('?')[0].split('/')[-1]
    
    try:
        isUidSuccess, uid = GetUidByRoomNumber(roomNumber)
    except Exception:
        ex_type, ex_val, _ = sys.exc_info()
        logger.error(f'{__PLUGIN_NAME}获取主播 <{uid}> 信息时发生错误')
        logger.error(f'{__PLUGIN_NAME}错误类型: {ex_type},错误值: {ex_val}')
        await followStreamerByShare.finish('关注失败: 连接错误')
    else:
        if isUidSuccess:
            successList, failList = await FollowStreamers(event, event.sender.user_id, [uid], 0)

            if successList:
                await followUpByShare.finish(f"关注主播成功: <{successList[0]}>")
            elif failList:
                await followUpByShare.finish(f"关注主播失败: <{failList[0]}> ")

        else:
            await followStreamerByShare.finish(f'关注失败: 非法uid')

followTelegramByShare = on_message(
    rule = regex('^\[CQ:json[\w\W]*"appid":100951776[\w\W]*www.bilibili.com\/bangumi\/play\/[\w\W]*') & privateMessageRule,
    permission=PRIVATE_FRIEND
)
@followTelegramByShare.handle()
async def telegramShareHandler(event: PrivateMessageEvent):
    '''响应用户分享番剧页面

    Args:
        event (PrivateMessageEvent): 消息事件
    '''
    sJson = event.get_message()[-1].get('data')
    data = json.loads(sJson['data'])
    epID = data['meta']['detail_1']['qqdocurl'].split('?')[0].split('/')[-1]
    epID = epID[2:]
    
    
    successList, failList = await FollowTelegram(event, event.sender.user_id, [epID], 0)
    
    if successList:
        await followUpByShare.finish(f"关注番剧成功: <{successList[0]}>")
    elif failList:
        await followUpByShare.finish(f"关注番剧失败: <{failList[0]}> ")

followByShareB23Url = on_message(
    rule = regex('^\[CQ:json[\w\W]*"appid":100951776[\w\W]*b23.tv[\w\W]*') & privateMessageRule,
    permission=PRIVATE_FRIEND
)
@followByShareB23Url.handle()
async def b23UrlShareHandler(event: PrivateMessageEvent):
    sJson = event.get_message()[-1].get('data')
    shortLink = re.search("https:\/\/b23.tv\/\w+", sJson['data'])
    b23Url = shortLink.group()
    
    msgType = 0
    userID = event.sender.user_id
    
    try:
        isSuccess, idType, id = parseB23Url(b23Url)
    except Exception:
        ex_type, ex_val, _ = sys.exc_info()
        logger.error(f'{__PLUGIN_NAME}解析短链接 <{b23Url}> 时发生错误')
        logger.error(f'{__PLUGIN_NAME}错误类型: {ex_type},错误值: {ex_val}')
        await followByShareB23Url.finish('关注失败: 连接错误')
    else:
        if isSuccess:
            if idType == 1:
                try:
                    isUidSuccess, uid = GetUidByRoomNumber(id)
                except Exception:
                    ex_type, ex_val, _ = sys.exc_info()
                    logger.error(f'{__PLUGIN_NAME}获取主播 <{uid}> 信息时发生错误')
                    logger.error(f'{__PLUGIN_NAME}错误类型: {ex_type},错误值: {ex_val}')
                    await followByShareB23Url.finish('关注失败: 连接错误')
                else:
                    if isUidSuccess:
                        successList, failList = await FollowStreamers(event, userID, [uid], msgType)
                        if successList:
                            await followByShareB23Url.finish(f"关注成功: {successList[0]}")
                        elif failList:
                            await followByShareB23Url.finish(f"关注失败: {failList[0]}")
                    else:
                        await followByShareB23Url.finish(f'关注失败: 非法uid')

            elif idType == 2:
                successList, failList = await FollowUp(event, userID, [uid], msgType)
                if successList:
                    await followByShareB23Url.finish(f"关注成功: {successList[0]}")
                elif failList:
                    await followByShareB23Url.finish(f"关注失败: {failList[0]}")

            elif idType == 3:
                successList, failList = await FollowTelegram(event, userID, [uid], msgType)
                if successList:
                    await followByShareB23Url.finish(f"关注成功: {successList[0]}")
                elif failList:
                    await followByShareB23Url.finish(f"关注失败: {failList[0]}")
        else:
            await followByShareB23Url.finish(f"关注失败: 非法短链接{b23Url}")

helpCommand = on_command("help", permission=ALL_PERMISSION, aliases={'帮助'})
@helpCommand.handle()
async def sendHelpMsg(event: MessageEvent):
    userID = event.sender.user_id
    helpMsg = ""
    with open(f'{PackagePath}/file/source/help.txt', 'r') as f:
        helpMsg = f.read()
    await helpCommand.finish(helpMsg)

publicBroacast = on_command("broacast", aliases={'广播'}, permission=permission.SUPERUSER)
@publicBroacast.handle()
async def sendBroacast(event: MessageEvent):
    userID = event.sender.user_id
    announcement = ""
    announcementPath = f'{PackagePath}/file/source/announcement.txt'
    if os.path.exists(announcementPath):
        with open(f'{PackagePath}/file/source/announcement.txt', 'r') as f:
            announcement = f.read()
            

        users = GetAllUser()
        await SendMsgToUsers(announcement, users)

        groups = GetAllGroup()
        await SendMsgToGroups(announcement, groups)

        await publicBroacast.finish("公告发送成功")
    else:
        logger.debug(f'{__PLUGIN_NAME}公告文件不存在')
        await publicBroacast.finish("公告发送失败: 公告文件不存在") 

scheduler = require("nonebot_plugin_apscheduler").scheduler
scheduler.add_job(CheckBiliStream, "interval", seconds=10, id="biliStream", misfire_grace_time=90)
scheduler.add_job(CheckUpUpdate, "interval", minutes=1, id="biliUp", misfire_grace_time=90)
scheduler.add_job(CheckTeleUpdate, "interval", minutes=5, id="biliTele", misfire_grace_time=90)

CheckDir()


import requests
import json
from typing import Tuple, List, Union
import sys
import os
import traceback
import nonebot
from nonebot.log import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from .basicFunc import *
__PLUGIN_NAME = "B站整合~视频"
baseUrl = 'https://api.bilibili.com/x/space/arc/search?mid={}&ps=30&tid=0&pn=1&keyword=&order=pubdate&jsonp=jsonp'
biliUserInfoUrl = 'https://api.bilibili.com/x/space/acc/info?mid={}&jsonp=jsonp'
header = {
    'User-Agent':'Mozilla/5.0 (Windows NT 6.1; rv2.0.1) Gecko/20100101 Firefox/4.0.1'
}
videoDir = f"{PackagePath}/file/up/"

# 视频
def GetLatestVideo(uid: str, lastPostTime: int) -> Tuple[bool, str, str, int, str]:
    """
    @description  :
    根据uid和时间戳, 返回元组，表示存在新视频或无新视频
    ---------
    @param  :
    uid: 查询新视频用户的uid
    lastPostTime: 文件中记录的最新视频的时间戳
    -------
    @Returns  :
    返回一个元组[是否更新，bv号，标题，发布时间戳，封面的链接]
    -------
    """
    
    response = requests.get(url = baseUrl.format(uid), headers=header)
    assert response.status_code == 200, '获取视频列表时连接出错, status_code = {}'.format(response.status_code)

    response = json.loads(response.text)
    #assert len(response['data']['list']['vlist']) != 0, '用户{}无发布视频'.format(uid)
    latestVideo = response['data']['list']['vlist'][0] if len(response['data']['list']['vlist']) != 0 else 0
    postTime = int(latestVideo['created']) if len(response['data']['list']['vlist']) else 0
    
    if postTime > lastPostTime:
        # 发现新视频
        title = latestVideo['title']
        bvID = latestVideo['bvid']
        picUrl = latestVideo['pic']
        return (True, bvID, title, postTime, picUrl)
    return (False, '', '', 0, '')

async def CheckUpUpdate():
    """
    @description  :
    检查关注UP主是否更新新视频，如果更新则通知用户并写入文件
    ---------
    @param  :
    -------
    @Returns  :
    -------
    """
    upFiles = os.listdir(videoDir)
    for filename in upFiles:
        with open(videoDir + '/' + filename, 'r+') as f:
            info = json.load(f)
            # upInfo = [upName, latestVideoTimeStamp, [userfollowers], [groupfollowers]]
            schedBot = nonebot.get_bot()

            try:
                res = GetLatestVideo(filename.split('.')[0], info[1])
                """
                res[0]: 是否更新
                res[1]: bv号
                res[2]: 视频标题
                res[3]: 发布时间(时间戳)
                res[4]: 封面链接
                """
                if res[0]:
                    logger.info(f'{__PLUGIN_NAME}检测到up主{info[0]}更新了视频')
                    textMsg = "【B站动态】\n <{}> 更新了视频\n标题: {}\n链接: https://www.bilibili.com/video/{}".format(info[0], res[2], res[1])
                    
                    for follower in info[2]:
                        logger.debug(f'{__PLUGIN_NAME}向用户{follower}发送更新通知')
                        await schedBot.send_msg(message=textMsg + MessageSegment.image(res[4]), user_id=follower)
                    
                    for group in info[3]:
                        await schedBot.send_msg(message=textMsg + MessageSegment.image(res[4]), group_id=group)
                    
                    info[1] = res[3]
                    f.seek(0)
                    f.truncate()
                    json.dump(info, f, ensure_ascii=False)

            except Exception as e:
                ex_type, ex_val, _ = sys.exc_info()
                exceptionMsg = '【错误报告】\n检测用户{}B站视频时发生错误\n错误类型: {}\n错误值: {}\n'.format(info[0], ex_type, ex_val)
                logger.error(f"{__PLUGIN_NAME}" + exceptionMsg + traceback.format_exc())


"""
bilibili视频更新关注命令
"""

def InitUpInfo(uid: str) -> Tuple[str, int]:
    """
    @description  :
    根据uid查询up主信息
    ---------
    @param  :
    uid：用户的uid
    -------
    @Returns  :
    返回一个元组
    [up主名字，最新视频的时间戳]
    -------
    """
    
    
    response = requests.get(url=biliUserInfoUrl.format(uid), headers=header)
    response = json.loads(response.text)
    if response['code'] == 0:
        userName = response['data']['name']
        res = GetLatestVideo(uid, 0)
        return (userName, res[3])
    else:
        return ('', '')

async def FollowModifyUpFile(uid: str, userID: int, type: int) -> Tuple[bool, str]:
    '''根据用户或群关注up，修改up文件

    Args:
        uid (str): up的uid
        userID (int): 用户的qq或群号
        type (int): 0-个人用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，信息]
    '''
    
    if uid.isdigit():
        upFile = f"{PackagePath}/file/up/{uid}.json"
        if os.path.exists(upFile):
            logger.debug(f"{__PLUGIN_NAME}up主文件{upFile}已经存在")

            with open(upFile, "r+") as f:
                upInfo: List = json.load(f)
                # upInfo = [upName, latestVideoTimeStamp, [userFollowers], [groupFollowers]]
                logger.debug(f"{__PLUGIN_NAME}正在读取up主{upInfo[0]}文件")
                if userID not in upInfo[2 + type]:
                    upInfo[2 + type].append(userID)
                    logger.debug(f"{__PLUGIN_NAME}用户/群{userID}关注up主{upInfo[0]}成功")
                    f.seek(0)
                    f.truncate()
                    json.dump(upInfo, f, ensure_ascii=False)
                    return (True, upInfo[0] + f"(uid: {uid})")
                else:
                    logger.debug(f"{__PLUGIN_NAME}用户{userID}已经关注了up{upInfo[0]}")
                    return (False, upInfo[0] + "(已关注)")
        else:
            logger.debug(f"{__PLUGIN_NAME}up主{uid}文件不存在")
            
            try:
                userName, latestTimeStamp = InitUpInfo(uid)
            except Exception:
                ex_type, ex_val, _ = sys.exc_info()
                logger.error(f'{__PLUGIN_NAME}获取up主{uid}信息时发生错误')
                logger.error(f'{__PLUGIN_NAME}错误类型: {ex_type},错误值: {ex_val}')
                return (False, uid + "(网络连接错误)")
            else:
                if userName:
                    upInfo = [userName, latestTimeStamp, [], []]
                    upInfo[2 + type].append(userID)

                    with open(upFile, "w+") as f:
                        json.dump(upInfo, f, ensure_ascii=False)
                    
                    logger.debug(f"{__PLUGIN_NAME}已创建up主{userName}的文件")
                    logger.debug(f"{__PLUGIN_NAME}用户{userID}关注up{upInfo[0]}成功")
                    return (True, upInfo[0] + f"(uid: {uid})")
                else:
                    logger.debug(f'{__PLUGIN_NAME}up{uid}不存在，请检查uid')
                    return (False, uid + "(uid错误)")
    else:
        return (False, uid + "(错误参数)")

async def UnfollowModifyUpFile(uid: str, userID: int, type: int) -> Tuple[bool, str]:
    '''根据个人用户或群取关up主，修改up文件

    Args:
        uid (str): up的uid
        userID (int): 用户的qq号/群号
        type (int): 0-个人用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，信息]
    '''
    
    if uid.isdigit():
        upFile = f"{PackagePath}/file/up/{uid}.json"
        if os.path.exists(upFile):
            with open(upFile, "r+") as f:
                upInfo: List = json.load(f)
                # upInfo = [upName, latestVideoTimeStamp, [userFollowers], [groupFollowers]]
                logger.debug(f"{__PLUGIN_NAME}正在读取用户{upInfo[0]}文件")
                if userID not in upInfo[2 + type]:
                    logger.debug(f'{__PLUGIN_NAME}用户{userID}未关注up主{uid}')
                    return (False, uid + "(未关注)")
                else:
                    upInfo[2 + type].remove(userID)
                    if upInfo[2] or upInfo[3]:
                        f.seek(0)
                        f.truncate()
                        json.dump(upInfo, f, ensure_ascii=False)
                    else:
                        logger.debug(f'{__PLUGIN_NAME}up主{upInfo[0]}已无人关注，将文件删除')
                        os.remove(upFile)
                        
                    logger.debug(f'{__PLUGIN_NAME}用户{userID}成功取关up主{upInfo[0]}')
                    return (True, upInfo[0]  + f"(uid: {uid})")
        else:
            logger.debug(f'{__PLUGIN_NAME}用户{userID}未关注up主{uid}')
            return (False, uid + "(未关注)")
    else:
        return (False, uid + "(错误参数)")

async def FollowUp(
    event: Union[PrivateMessageEvent, GroupMessageEvent], 
    id: int, 
    uidList: List[str],
    type: int
    ) -> List[List[str]]:
    '''个人用户/群关注up主

    Args:
        event (Union[PrivateMessageEvent, GroupMessageEvent]): 消息事件
        id (int): qq号/群号
        uidList (List[str]): up主的uid
        type (int): 0-个人用户，1-群        

    Returns:
        List[List[str]]: [是否成功，信息]
    '''
    userFile = f"{PackagePath}/file/{'user' if type == 0 else 'group'}/{id}.json"
    successList = []
    failList = []

    for uid in uidList:
        isSuccess, s = await FollowModifyUpFile(uid, id, type)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)

    if os.path.exists(userFile):
        await FollowModifyUserFile(userFile, successList, 2)
    else:
        logger.debug(f'{__PLUGIN_NAME}用户文件{userFile}不存在, 准备创建')
        name = event.sender.nickname

        if type == 1:
            bot = get_bot()
            groupInfo = await bot.get_group_info(group_id=id)
            name = groupInfo["group_name"]

        await createUserFile(userFile, name, ups=successList)
    
    return [successList, failList]

async def UnfollowUp(
    event: Union[PrivateMessageEvent, GroupMessageEvent], 
    id: int, 
    uidList: List[str],
    type: int
    ) -> List[List[str]]:
    '''个人用户/群取关up主

    Args:
        event (Union[PrivateMessageEvent, GroupMessageEvent]): 消息事件
        id (int): qq号/群号
        uidList (List[str]): 取关的up主
        type (int): 0-个人用户，1-群

    Returns:
        List[List[str]]: [是否成功，信息]
    '''
    userFile = f"{PackagePath}/file/{'user' if type == 0 else 'group'}/{id}.json"
    successList = []
    failList = []

    for uid in uidList:
        isSuccess, s = await UnfollowModifyUpFile(uid, id, type)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)
    
    if os.path.exists(userFile):
        await UnfollowModifyUserFile(userFile, successList, 2)
    else:
        logger.debug(f'{__PLUGIN_NAME}用户文件{userFile}不存在, 准备创建')
        name = event.sender.nickname

        if type == 1:
            bot = get_bot()
            groupInfo = await bot.get_group_info(group_id=id)
            name = groupInfo["group_name"]
        
        await createUserFile(userFile, name)

    return [successList, failList]


import requests
import json
from typing import Tuple, List, Union
import sys
import traceback
import nonebot
import os
from nonebot.log import logger
from nonebot.adapters.onebot.v11 import Message, MessageSegment
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from .basicFunc import *
__PLUGIN_NAME = "B站整合~直播"
baseUrl = 'https://api.bilibili.com/x/space/arc/search?mid={}&ps=30&tid=0&pn=1&keyword=&order=pubdate&jsonp=jsonp'
biliUserInfoUrl = 'https://api.bilibili.com/x/space/acc/info?mid={}&jsonp=jsonp'
biliLiveInfoUrl = 'https://api.live.bilibili.com/xlive/web-room/v2/index/getRoomPlayInfo?room_id={}&protocol=0,1&format=0,1,2&codec=0,1&qn=0&platform=web&ptype=8'
header = {
    'User-Agent':'Mozilla/5.0 (Windows NT 6.1; rv2.0.1) Gecko/20100101 Firefox/4.0.1'
}

streamDir = f'{PackagePath}/file/stream/'

def GetBiliStream(uid: str) -> Tuple[bool, str, str]:
    """
    @description  :
    获取bilibili直播间信息
    ---------
    @param  :
    uid: 主播的uid，不是直播间号码
    -------
    @Returns  :
    返回一个元组
    [是否正在直播，直播间标题，直播间封面链接]
    -------
    """
    
    response = requests.get(url=biliUserInfoUrl.format(uid), headers=header)
    assert response.status_code == 200, '获取直播间信息时连接错误, status_code = {}'.format(response.status_code)

    response = json.loads(response.text)
    if response['code'] == 0:
        liveRoom = response['data']['live_room']
        liveStatus = liveRoom['liveStatus']
        title = liveRoom['title']
        coverURL = liveRoom['cover']

        if liveStatus == 1:
            return (True, title, coverURL)
    return (False, '', '')

async def CheckBiliStream():
    """
    @description  :
    检查streamDir中所有主播的开播状态
    如果关注的主播开播，则通知所有关注的用户
    如果主播下播了，则写入文件
    ---------
    @param  :
    -------
    @Returns  :
    -------
    """
    #logger.info(f'[{__PLUGIN_NAME}]开始检测开播状态')
    streamerFiles = os.listdir(streamDir)
    for filename in streamerFiles:
        with open(streamDir + '/' + filename, 'r+') as f:
            #logger.info(f'[{__PLUGIN_NAME}]打开文件{filename}！')
            info = json.load(f)
            # [username, isStreaming, roomURL, [followers]]
            shouldUpdated = False
            schedBot = nonebot.get_bot()
            
            try:
                isStreaming, title, coverURL = GetBiliStream(filename.split('.')[0])
                if isStreaming and not info[1]:
                    logger.info(f'[{__PLUGIN_NAME}]检测到主播{info[0]}已开播！')
                    shouldUpdated = True
                    info[1] = True
                    textMsg = '【直播动态】\n<{}>正在直播!\n标题: {}\n链接: {}'.format(info[0], title, info[2])
                    
                    reportedMsg = Message([
                        MessageSegment(type='text', data={'text':textMsg}),
                        MessageSegment(type='image', data={'file':coverURL})
                    ])
                    
                    logger.info(f'[{__PLUGIN_NAME}]向粉丝发送开播通知')
                    for follower in info[3]:
                        await schedBot.send_msg(message=reportedMsg, user_id=follower)
                    for groupFollower in info[4]:
                        await schedBot.send_msg(message=reportedMsg, group_id=groupFollower)
                elif not isStreaming and info[1]:
                    logger.info(f"检测到主播{info[0]}已下播！")
                    shouldUpdated = True
                    info[1] = False
            except Exception as e:
                ex_type, ex_val, _ = sys.exc_info()
                exceptionMsg = '【错误报告】\n检测用户{}B站直播时发生错误\n错误类型: {}\n错误值: {}\n'.format(info[0], ex_type, ex_val)
                logger.error(f"{__PLUGIN_NAME}\n" + exceptionMsg + traceback.format_exc())
            
            if shouldUpdated:
                f.seek(0)
                f.truncate()
                json.dump(info, f, ensure_ascii=False) 

def InitStreamerInfo(uid: str) -> Tuple[str, str]:
    """
    @description  :
    根据uid，返回直播间信息
    ---------
    @param  :
    uid： 用户的uid
    -------
    @Returns  :
    返回一个元组
    [用户名，直播间链接]
    -------
    """
    
    response = requests.get(url=biliUserInfoUrl.format(uid), headers=header)
    response = json.loads(response.text)
    if response['code'] == 0:
        userName = response['data']['name']
        if response['data']["live_room"]:
            roomURL = response['data']['live_room']['url']
        else:
            roomURL = ""
        logger.info(f"获取到uid为{uid}的用户名为{userName}, 直播间链接为{roomURL}")
        return (userName, roomURL)
    else:
        return ('', '')

def GetUidByRoomNumber(roomNumber: str) -> Tuple[bool, str]:
    """
    @description  :
    根据直播间号获得uid
    ---------
    @param  :
    roomNumber: 房间号
    -------
    @Returns  :
    返回一个元组
    [isSuccess, uid | '']
    -------
    """
    
    res = requests.get(url=biliLiveInfoUrl.format(roomNumber), headers=header)
    assert res.status_code == 200, "获取直播间信息时出现连接错误"
    res = json.loads(res.text)
    if res['code'] == 0:
        uid = res['data']['uid']
        return (True, str(uid))
    else:
        return (False, '')
    
async def FollowModifyStreamerFile(uid: str, userID: int, type: int) -> Tuple[bool, str]:
    '''根据用户/群关注主播，修改主播文件

    Args:
        uid (str): 主播的uid
        userID (int): 用户的uid或群号
        type (int): 0-用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，主播名(uid) | 主播uid(失败原因)]
    '''
    
    if not uid.isdigit():
        logger.debug(f'{__PLUGIN_NAME}存在错误参数{uid}')
        return (False, uid + "(错误参数)")     
    streamerFile = f"{PackagePath}/file/stream/{uid}.json"
    if os.path.exists(streamerFile):
        logger.debug(f"{__PLUGIN_NAME}主播{uid}文件已经存在")
        with open(streamerFile, "r+") as f:
            streamerInfo: List = json.load(f)
            # streamerInfo = [streamerName, isStreaming, roomURL, [privateFollowers], [groupFollowers]]
            if userID not in streamerInfo[3 + type]:
                streamerInfo[3 + type].append(userID)
                if type == 0:
                    logger.debug(f"{__PLUGIN_NAME}用户{userID}关注主播{streamerInfo[0]}成功")
                else:
                    logger.debug(f'{__PLUGIN_NAME}群{userID}关注主播{streamerInfo[0]}成功')
                    
                f.seek(0)
                f.truncate()
                json.dump(streamerInfo, f, ensure_ascii=False)
                return (True, streamerInfo[0] + f"(uid: {uid})")
            else:
                logger.debug(f"{__PLUGIN_NAME}用户/群{userID}已经关注了主播{streamerInfo[0]}")
                return (False, streamerInfo[0] + "(已关注)")
                
    else:
        logger.debug(f"{__PLUGIN_NAME}用户{uid}文件不存在")
        try:
            userName, roomURL = InitStreamerInfo(uid)
        except Exception:
            ex_type, ex_val, _ = sys.exc_info()
            logger.error(f'{__PLUGIN_NAME}获取主播{uid}信息时发生错误')
            logger.error(f'{__PLUGIN_NAME}错误类型: {ex_type},错误值: {ex_val}')
            return (False, uid + "(网络连接错误)")
        else:
            if userName and roomURL:
                streamerInfo = [userName, False, roomURL, [], []]
                streamerInfo[3 + type].append(userID)
                with open(streamerFile, "w+") as f:
                    json.dump(streamerInfo, f, ensure_ascii=False)
                logger.debug(f"{__PLUGIN_NAME}用户/群{userID}关注主播{streamerInfo[0]}成功")
                return (True, streamerInfo[0] + f"(uid: {uid})")
            elif userName:
                logger.debug(f'{__PLUGIN_NAME}主播{userName}未开通直播间')
                return (False, uid + "(该用户未开通直播间)")
            else:
                logger.debug(f'{__PLUGIN_NAME}主播{uid}不存在')
                return (False, uid + "(非法uid)")
                

async def UnfollowModifyStreamerFile(uid: str, userID: int, type: int) -> Tuple[bool, str]:
    '''根据用户取关主播，修改主播文件

    Args:
        uid (str): 主播的uid
        userID (int):QQ号/群号
        type (int): 0-个人用户，1-群

    Returns:
        Tuple[bool, str]: [是否成功，主播名 | 失败原因]
    '''
    if uid.isdigit():
        streamerFile = f"{PackagePath}/file/stream/{uid}.json"
        if os.path.exists(streamerFile):
            with open(streamerFile, "r+") as f:
                streamerInfo: List = json.load(f)
                # streamerInfo = [streamerName, isStreaming, roomURL, [privateFollowers], [groupFollowers]]
                if userID not in streamerInfo[3 + type]:
                    logger.debug(f'{__PLUGIN_NAME}用户{userID}未关注主播{uid}')
                    return (False, uid + "(未关注)")
                else:
                    streamerInfo[3 + type].remove(userID)
                    if streamerInfo[3] or streamerInfo[4]:
                        f.seek(0)
                        f.truncate()
                        json.dump(streamerInfo, f, ensure_ascii=False)
                    else:
                        logger.debug(f'{__PLUGIN_NAME}主播{streamerInfo[0]}已无人关注，将文件删除')
                        os.remove(streamerFile)
                        
                    logger.debug(f'{__PLUGIN_NAME}用户{userID}成功取关主播{streamerInfo[0]}')
                    return (True, streamerInfo[0] + f"(uid: {uid})")
        else:
            logger.debug(f'{__PLUGIN_NAME}用户{userID}未关注主播{uid}')
            return (False, uid + "(未关注)")
    else:
        return (False, uid + "(错误参数)")

async def FollowStreamers(
    event: Union[PrivateMessageEvent, GroupMessageEvent], 
    id: int, 
    uidList: List[str],
    userType: int
    ) -> List[List[str]]:
    '''用户/群对主播进行关注

    Args:
        event (Union[PrivateMessageEvent, GroupMessageEvent]): 消息事件
        id (int): 用户qq或群号
        uidList (List[str]): 关注的主播uid
        userType (int): 0-用户, 1-群

    Returns:
        List[List[str]]: [[关注成功], [关注失败]]
    '''
    userFile = f"{PackagePath}/file/{'user' if userType == 0 else 'group'}/{id}.json"
    successList = []
    failList = []

    for uid in uidList:
        isSuccess, s = await FollowModifyStreamerFile(uid, id, userType)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)

    if os.path.exists(userFile):
        await FollowModifyUserFile(userFile, successList, 1)
    else:
        logger.debug(f'{__PLUGIN_NAME}用户文件{userFile}不存在, 准备创建')
        name = event.sender.nickname
        if userType == 1:
            bot = get_bot()
            groupInfo = await bot.get_group_info(group_id=id)
            name = groupInfo["group_name"]

        await createUserFile(userFile, name, streamers=successList)
    
    return [successList, failList]


async def UnfollowStreamers(
    event: Union[PrivateMessageEvent, GroupMessageEvent], 
    id: int, 
    uidList: List[str],
    type: int
    ) -> List[List[str]]:
    '''用户/群对主播取关

    Args:
        event (Union[PrivateMessageEvent, GroupMessageEvent]): 消息事件
        id (int): 用户qq/群号
        uidList (List[str]): 取关主播列表
        type (int): 0-用户, 1-群

    Returns:
        List[List[str]]: [成功列表，失败列表]
    '''
    userFile = f"{PackagePath}/file/{'user' if type == 0 else 'group'}/{id}.json"
    successList = []
    failList = []

    for uid in uidList:
        isSuccess, s = await UnfollowModifyStreamerFile(uid, id, type)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)
    
    if os.path.exists(userFile):
        await UnfollowModifyUserFile(userFile, successList, 1)
    else:
        logger.debug(f'{__PLUGIN_NAME}用户文件{userFile}不存在, 准备创建')
        name = event.sender.nickname
        if type == 1:
            bot = get_bot()
            groupInfo = await bot.get_group_info(group_id=id)
            name = groupInfo["group_name"]
        await createUserFile(userFile, name)
    
    return [successList, failList]


import requests
import json
from typing import Tuple, List, Union
import sys
import os
import traceback
import nonebot
from nonebot.log import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from .basicFunc import *

__PLUGIN_NAME = "B站整合~影视/番剧"
biliTeleInfoUrl = 'https://api.bilibili.com/pgc/web/season/section?season_id={}'
getSeasonIDAPI = 'https://api.bilibili.com/pgc/view/web/season?ep_id={}'
getEpisodesAPI = 'https://api.bilibili.com/pgc/web/season/section?season_id={}'
telegramDir = f'{PackagePath}/file/telegram/'

header = {
    'User-Agent':'Mozilla/5.0 (Windows NT 6.1; rv2.0.1) Gecko/20100101 Firefox/4.0.1'
}

FollowTelegramFile = '/root/project/NoneBot/TDK_Bot/src/file/FollowedTelegram.json'

def GetTelegramInfo(seasonID: str, index: int) -> Tuple[bool, int, str, str, str]:
    """
    @description  :
    获取影视区作品的更新情况
    ---------
    @param  :
    seasonID: 影视区作品的id
    index: 文件记录中的最新一集
    -------
    @Returns  :
    返回一个元组
    (是否更新, 最新集数, 最新集标题, 最新集链接, 封面链接)
    -------
    """
    
    response = requests.get(url=getEpisodesAPI.format(seasonID), headers=header)
    assert response.status_code == 200, '查询影视{}信息连接错误，status_code = {}'.format(response.status_code)

    response = json.loads(response.text)

    if response['code'] == 0:
        episodes = response['result']['main_section']['episodes']
        if len(episodes) > index:
            # 影视有更新
            latestEpisode = episodes[-1]
            coverURL = latestEpisode['cover']
            title = latestEpisode['long_title']
            playURL = latestEpisode['share_url']
            return (True, len(episodes), title, playURL, coverURL)
        else:
            return (False, 0, '', '', '')
    else:
        logger.debug(f"[{__PLUGIN_NAME}]查询的影视片不存在")
        return (False, 0, '', '', '')

def GetSeasonIDByEpid(epID: str) -> Tuple[bool, str, str, int]:
    """
    @description  :
    根据单集的epid，获取整季的seasonID以及名字
    ---------
    @param  :
    epid: 单集的epid
    -------
    @Returns  :
    返回一个元组
    [isSuccess, seasonID, seasonTitle, latestIndex]
    
    -------
    """
    res = requests.get(url=getSeasonIDAPI.format(epID), headers=header)
    assert res.status_code == 200, f'获取seasonID时发生连接错误，status_code = {res.status_code}'

    res = json.loads(res.text)
    if res['code'] == 0:
        seasonID = str(res['result']['season_id'])
        seasonTitle = res['result']['season_title']
        latestIndex = len(res['result']['episodes'])

        return (True, seasonID, seasonTitle, latestIndex)
    else:
        logger.debug(f'{__PLUGIN_NAME}获取seasonID失效，请检查epid')
        return (False, '', '', 0)

    
async def CheckTeleUpdate():
    """
    @description  :
    检查文件中的每一个影视节目是否更新，如果更新则向用户发送通知，并且更新文件
    ---------
    @param  :
    无
    -------
    @Returns  :
    无
    -------
    """
    telegramFiles = os.listdir(telegramDir)
    for filename in telegramFiles:
        with open(telegramDir + '/' + filename, 'r+') as f:
            info = json.load(f)
            # [telegramName, latestIndex, [followers]]
            schedBot = nonebot.get_bot()
            shouldUpdated = False
            epID = filename.split('.')[0]
            
            try:
                res = GetTelegramInfo(epID, info[1])
                if res[0]:
                    logger.info(f'{__PLUGIN_NAME}检测到影视剧{info[0]}更新')
                    
                    shouldUpdated = True
                    info[1] = res[1]
                    textMsg = "【B站动态】\n《{}》已更新第{}集\n标题: {}\n链接: {}\n".format(
                        info[0], res[1], res[2], res[3]
                    )
                    coverMsg = MessageSegment.image(res[4])
                    logger.info(f'{__PLUGIN_NAME}向关注用户发送更新通知')
                    
                    for follower in info[2]:
                        await schedBot.send_msg(message=textMsg + coverMsg, user_id=follower)
                    
                    for group in info[3]:
                        await schedBot.send_msg(message=textMsg + coverMsg, group_id=group)
                    logger.info(f"[{__PLUGIN_NAME}]通知用户节目《{info[0]}》已更新第{res[1]}集")
            except Exception as e:
                ex_type, ex_val, _ = sys.exc_info()
                exceptionMsg = '【错误报告】\n检测节目《{}》时发生错误\n错误类型: {}\n错误值: {}\n'.format(info[0], ex_type, ex_val)
                logger.error(f"{__PLUGIN_NAME}\n" + exceptionMsg + traceback.format_exc())
            else:
                if shouldUpdated:
                    f.seek(0)
                    f.truncate()
                    json.dump(info, f, ensure_ascii=False)
                    logger.info(f"[{__PLUGIN_NAME}]文件FollowTelegramFile已更新！")

async def FollowModifyTelegramFile(epID: str, userID: int, type: int) -> Tuple[bool, str]:
    '''根据用户关注节目，修改节目的文件

    Args:
        epID (str): 节目的id
        userID (int): 用户的qq号/群号
        type (int): 0-个人用户，1-群号

    Returns:
        Tuple[bool, str]: [是否成功，信息]
    '''

    if not epID.isdigit():
        logger.debug(f'{__PLUGIN_NAME}存在错误参数{epID}')
        return (False, epID + "(错误参数)")
    try:
        res = GetSeasonIDByEpid(epID)
    except Exception:
        ex_type, ex_val, _ = sys.exc_info()
        exceptionMsg = '【错误报告】\n根据epID:{}获取seasonID时发生错误\n错误类型: {}\n错误值: {}\n'.format(epID, ex_type, ex_val)
        logger.error(f"{__PLUGIN_NAME}\n" + exceptionMsg + traceback.format_exc())
        return (False, epID + "(网络错误)")
    else:
        if res[0]:
            telegramFile = f"{PackagePath}/file/telegram/{res[1]}.json"
            if os.path.exists(telegramFile):
                logger.debug(f'{__PLUGIN_NAME}节目{res[2]}文件已经存在')
                with open(telegramFile) as f:
                    telegramInfo: List = json.load(f)
                    # telegramInfo = [telegramTitle, latestIndex, [userFollowers], [groupFollowers]]
                    logger.debug(f'{__PLUGIN_NAME}正在读取节目文件{telegramFile}')

                    if userID not in telegramInfo[2 + type]:
                        telegramInfo[2 + type].append(userID)
                        logger.debug(f'{__PLUGIN_NAME}用户{userID}关注节目{res[2]}成功')
                        f.seek(0)
                        f.truncate()
                        json.dump(telegramFile, f, ensure_ascii=False)
                        return (True, res[2] + f"(seasonID: {res[1]})")
                    else:
                        logger.debug(f'{__PLUGIN_NAME}用户{userID}已关注节目{res[2]}')
                        return (False, res[2] + "(已关注)")
            else:
                logger.debug(f'{__PLUGIN_NAME}节目{res[2]}文件不存在')
                telegramInfo = [res[2], res[3], [], []]
                telegramInfo[2 + type].append(userID)

                with open(telegramFile, "w+") as f:
                    json.dump(telegramInfo, f, ensure_ascii=False)
                logger.debug(f'{__PLUGIN_NAME}已创建节目{res[2]}文件')
                logger.debug(f'{__PLUGIN_NAME}用户{userID}关注主播{res[2]}成功')
                return (True, res[2] + f"(seasonID: {res[1]})")       
        else:
            logger.debug(f'{__PLUGIN_NAME}')
            return (False, epID + "(错误的epID)")
        

async def UnfollowModifyTelegramFile(seasonID: str, userID: int, type: int) -> Tuple[bool, str]:
    '''根据用户/群取关节目，修改节目文件

    Args:
        seasonID (str): 节目的ID
        userID (int): 用户qq号/群号     
        type (int): 0-个人用户，1-群号

    Returns:
        Tuple[bool, str]: [是否成功, 信息]
    '''

    if not seasonID.isdigit():
        return (False, seasonID + "(错误参数)")
    
    telegramFile = f"{PackagePath}/file/telegram/{seasonID}.json"
    if os.path.exists(telegramFile):
        with open(telegramFile, "r+") as f:
            telegramInfo: List = json.load(f)
            # telegramInfo = [telegramTitle, latestIndex, [userFollowers], [groupFollowers]]
            logger.debug(f'{__PLUGIN_NAME}正在读取节目{telegramInfo[0]}文件')
            if userID not in telegramInfo[2 + type]:
                logger.debug(f'{__PLUGIN_NAME}用户{userID}未关注节目{telegramInfo[0]}')
                return (False, seasonID + "未关注")
            else:
                telegramInfo[2 + type].remove(userID)
                if telegramInfo[2] or telegramInfo[3]:
                    f.seek(0)
                    f.truncate()
                    json.dump(telegramInfo, f, ensure_ascii=False)
                else:
                    logger.debug(f'{__PLUGIN_NAME}节目{telegramInfo[0]}已经无人关注，将文件删除')
                    os.remove(telegramFile)

                logger.debug(f'{__PLUGIN_NAME}用户{userID}取关节目{telegramInfo[0]}成功')
                return (True, telegramInfo[0] + f"(seasonID: {seasonID})")
    else:
        logger.debug(f'{__PLUGIN_NAME}用户{userID}未关注节目{seasonID}')
        return (False, seasonID + "(未关注)")

async def FollowTelegram(
    event: Union[PrivateMessageEvent, GroupMessageEvent], 
    id: int, 
    epIDs: List[str],
    type: int
    ) -> List[List[str]]:
    '''个人用户/群关注番剧

    Args:
        event (Union[PrivateMessageEvent, GroupMessageEvent]): 消息事件
        id (int): qq号/群号
        epIDs (List[str]): 关注的番剧号
        type (int): 0-个人用户，1-群

    Returns:
        List[List[str]]: [是否成功，信息]
    '''
    userFile = f"{PackagePath}/file/{'user' if type == 0 else 'group'}/{id}.json"
    successList = []
    failList = []

    for epID in epIDs:
        if epID[0:2] != 'ep':
            failList.append(epID + "(错误参数)")
        else:
            epID = epID[2:]
            isSuccess, s = await FollowModifyTelegramFile(epID, id, type)
            if isSuccess:
                successList.append(s)
            else:
                failList.append(s)
    
    if os.path.exists(userFile):
        await FollowModifyUserFile(userFile, successList, 3)
    else:
        logger.debug(f'{__PLUGIN_NAME}用户文件{userFile}不存在, 准备创建')
        name = event.sender.nickname
        if type == 1:
            bot = get_bot()
            groupInfo = await bot.get_group_info(group_id=id)
            name = groupInfo["group_name"]
        await createUserFile(userFile, name, telegrams=successList)    

    return [successList, failList] 

async def UnfollowTelegram(
    event: Union[PrivateMessageEvent, GroupMessageEvent], 
    id: int, 
    seasonIDs: List[str],
    type: int
    ) -> List[List[str]]:
    '''个人用户/群取关番剧

    Args:
        event (Union[PrivateMessageEvent, GroupMessageEvent]): 消息事件
        id (int): qq号/群号
        epIDs (List[str]): 取关的番剧号
        type (int): 0-个人用户，1-群

    Returns:
        List[List[str]]: [是否成功，信息]
    '''
    userFile = f"{PackagePath}/file/{'user' if type == 0 else 'group'}/{id}.json"
    successList = []
    failList = []

    for seasonID in seasonIDs:
        isSuccess, s = await UnfollowModifyTelegramFile(seasonID, id, type)
        if isSuccess:
            successList.append(s)
        else:
            failList.append(s)
    
    if os.path.exists(userFile):
        await UnfollowModifyUserFile(userFile, successList, 3)
    else:
        logger.debug(f'{__PLUGIN_NAME}用户文件{userFile}不存在, 准备创建')
        name = event.sender.nickname
        if type == 1:
            bot = get_bot()
            groupInfo = await bot.get_group_info(group_id=id)
            name = groupInfo["group_name"]
        await createUserFile(userFile, name)
    return [successList, failList]                 
                

            
            

        
    
            



from typing import List, Tuple
import json
import re
import requests
import os
from nonebot import get_bot
from nonebot.log import logger
from os.path import abspath, dirname

PackagePath =  dirname(abspath(__file__))

__PLUGIN_NAME = "[B站整合~基础]"

async def createUserFile(userFile: str, nickName: str, streamers: List[str]=[], ups: List[str]=[], telegrams: List[str]=[]):
    '''创建用户/群文件

    Args:
        userFile (str): 文件路径
        nickName (str): 用户名/群名
        streamers (List[str], optional): 关注的主播. Defaults to [].
        ups (List[str], optional): 关注的up主. Defaults to [].
        telegrams (List[str], optional): 关注的番剧. Defaults to [].
    '''
    
    with open(userFile, 'w+') as f:
            userInfo = [nickName, streamers, ups, telegrams]
            json.dump(userInfo, f, ensure_ascii=False)
    logger.debug(f'{__PLUGIN_NAME}用户文件{userFile}创建成功')

async def FollowModifyUserFile(userFile: str, successList: List[str], type: int):
    """
    @description  :
    根据用户的关注操作，修改用户/群文件
    ---------
    @param  :
    userFile:       用户/群的文件名
    successList:    成功关注的列表
    type:           1 - 主播, 2 - up主, 3 - 节目
    -------
    @Returns  :
    无返回值
    -------
    """
    with open(userFile, "r+") as f:
        userInfo = json.load(f)
        userInfo[type] += successList
        # [nickname, [streamer], [up], [telegrams]]
        f.seek(0)
        f.truncate()
        json.dump(userInfo, f, ensure_ascii=False)
        logger.debug(f'{__PLUGIN_NAME}用户{userInfo[0]}文件更新成功')

async def UnfollowModifyUserFile(userFile: str, successList: List[str], type: int):
    """
    @description  :
    根据用户的取关操作，修改用户文件
    ---------
    @param  :
    userFile:       用户的文件名
    successList:    成功取关的列表
    type:           1 - 主播, 2 - up主, 3 - 节目
    -------
    @Returns  :
    无返回值
    -------
    """
    with open(userFile, "r+") as f:
        userInfo = json.load(f)
        userInfo[type] = list(set(userInfo[type]) - set(successList))
            # [nickname, [streamer], [up], [telegrams]]
        f.seek(0)
        f.truncate()
        json.dump(userInfo, f, ensure_ascii=False)

def parseB23Url(url: str) -> Tuple[bool, int, str]:
    """
    @description  :
    对b23.tv短链接进行转换
    ---------
    @param  :
    url: b23.tv类型的短链接
    -------
    @Returns  :
    返回一个元组
    [isSuccess, type, id]
    type: 1-直播房间号;2-up主uid;3-节目的epid
    -------
    """ 
    API = 'https://duanwangzhihuanyuan.bmcx.com/web_system/bmcx_com_www/system/file/duanwangzhihuanyuan/get/?ajaxtimestamp=1646565831920'
    payload = {'turl': url}
    res = requests.post(url=API, data=payload)
    assert res.status_code == 200, f'转换短链接时发生异常，status_code = {res.status_code}'

    if re.search("live.bilibili.com", res.text):
        roomNumber = re.search("live.bilibili.com/\d+", res.text)
        roomNumber = roomNumber.group()
        logger.debug(f'{__PLUGIN_NAME}短链接{url}是直播间链接,房间号为{roomNumber[18:]}')
        return (True, 1, roomNumber[18:])
    elif re.search("space.bilibili.com", res.text):
        uid = re.search("space.bilibili.com/\d+", res.text)
        uid = uid.group()
        logger.debug(f'{__PLUGIN_NAME}短链接{url}是个人空间,uid为{uid[19:]}')
        return (True, 2, uid[19:])
    elif re.search("www.bilibili.com/bangumi/play/ep", res.text):
        epid = re.search("www.bilibili.com/bangumi/play/ep\d+", res.text)
        epid = epid.group()
        logger.debug(f'{__PLUGIN_NAME}短链接{url}是番剧播放页面,epid为{epid[32:]}')
        return (True, 3, epid[32:])
    
    return (False, 0, '')

def GetAllUser() -> List[str]:
    """
    @description  :
    获取所有用户
    ---------
    @param  :
    无
    -------
    @Returns  :
    返回所有用户qq号组成的列表
    -------
    """
    
    users = os.listdir(f'{PackagePath}/file/user')
    result = []
    for user in users:
        userID = user.split('.')[0]
        result.append(userID)
    return result

def GetAllGroup() -> List[str]:
    """
    @description  :
    获取所有群
    ---------
    @param  :
    无
    -------
    @Returns  :
    返回所有群号组成的列表
    -------
    """
    
    users = os.listdir(f'{PackagePath}/file/group')
    result = []
    for user in users:
        userID = user.split('.')[0]
        result.append(userID)
    return result

async def SendMsgToUsers(msg: str, users: List[str]):
    """
    @description  :
    向所有用户发送公告
    ---------
    @param  :
    msg: 内容
    users: 用户的qq列表
    -------
    @Returns  :
    无
    -------
    """
    
    bot = get_bot()
    for user in users:
        await bot.send_msg(message=msg, user_id = user)
    
async def SendMsgToGroups(msg: str, groups: List[str]):
    '''向所有群组发送公告

    Args:
        msg (str): 公告内容
        groups (List[str]): 群组列表
    '''
    
    bot = get_bot()
    for group in groups:
        await bot.send_msg(message=msg, group_id = group)

def CheckDir():
    """
    @description  :
    检查插件运行所需要的文件目录是否存在，不存在则创建
    ---------
    @param  :
    
    -------
    @Returns  :
    
    -------
    """
    
    baseDir = f'{PackagePath}/file/'

    if not os.path.exists(baseDir + 'source'):
        logger.debug(f'{__PLUGIN_NAME}source文件夹不存在')
        os.makedirs(baseDir + 'source')
        
    if not os.path.exists(baseDir + 'stream'):
        logger.debug(f'{__PLUGIN_NAME}stream文件夹不存在')
        os.makedirs(baseDir + 'stream')
    
    if not os.path.exists(baseDir + 'telegram'):
        logger.debug(f'{__PLUGIN_NAME}telegram文件夹不存在')
        os.makedirs(baseDir + 'telegram')
    
    if not os.path.exists(baseDir + 'up'):
        logger.debug(f'{__PLUGIN_NAME}up文件夹不存在')
        os.makedirs(baseDir + 'up')
        
    if not os.path.exists(baseDir + 'user'):
        logger.debug(f'{__PLUGIN_NAME}user文件夹不存在')
        os.makedirs(baseDir + 'user')
    
    if not os.path.exists(baseDir + 'group'):
        logger.debug(f'{__PLUGIN_NAME}group文件夹不存在')
        os.makedirs(baseDir + 'group')




from nonebot.adapters.onebot.v11 import Message, MessageSegment, MessageEvent, Event
from nonebot.adapters.onebot.v11.event import GroupMessageEvent, PrivateMessageEvent
from nonebot.log import logger

from nonebot.rule import Rule
async def isGroupMessage(event: Event) -> bool:
    if event.get_type() != "message":
        return False
    if not isinstance(event, GroupMessageEvent):
        return False
    return True

async def isPrivateMessage(event: Event) -> bool:
    if event.get_type() != "message":
        return False
    if not isinstance(event, PrivateMessageEvent):
        return False
    return True

groupMessageRule = Rule(isGroupMessage)
privateMessageRule = Rule(isPrivateMessage)


from setuptools import setup, find_packages
import sys
import os

setup(
    name="nonebot_plugin_bilibilibot",
    version="2.0.1",
    author="TDK",
    author_email="tdk1969@foxmail.com",
    description="基于Nonebot的bilibili通知插件，可将up主，主播以及番剧的更新/直播动态推送到QQ",
    long_description=open("README.rst", "r").read(),
    license="GNU",
    url="https://github.com/TDK1969/nonebot_plugin_bilibilibot",
    packages=['nonebot_plugin_bilibilibot'],
    install_requires=[
        "requests",
        "nonebot_plugin_apscheduler",
        "nonebot_adapter_onebot"
    ],
    keywords='nonebot',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],

)